<?php


require_once __DIR__ . './vendor/autoload.php';

use GuzzleHttp\Client;
use GuzzleHttp\Psr7\MultipartStream;

// The description of the authorization method is available here: https://developers.sumsub.com/api-reference/#app-tokens 
define("SUMSUB_SECRET_KEY", "lIUl0p8OpzjLFS3ddELS9IeuLlvlG4dh"); // Example: Hej2ch71kG2kTd1iIUDZFNsO5C1lh5Gq
define("SUMSUB_APP_TOKEN", "prd:QptvpSYHvJsVdeSda6zLCa81.FRRXaaiQkITEgZeBAWgZETFknv2YPaMq"); // Example: sbx:uY0CgwELmgUAEyl4hNWxLngb.0WSeQeiYny4WEqmAALEAiK2qTC96fBad
define("SUMSUB_TEST_BASE_URL", "https://api.sumsub.com");
//Please don't forget to change token and secret key values to production ones when switching to production

class AppTokenGuzzlePhpExample
{
    public function createApplicant($externalUserId, $levelName)
        // https://developers.sumsub.com/api-reference/#creating-an-applicant
    {
        $requestBody = [
            'externalUserId' => $externalUserId
            ];

        $url = '/resources/applicants?levelName=' . $levelName;
        $request = new GuzzleHttp\Psr7\Request('POST', SUMSUB_TEST_BASE_URL . $url);
        $request = $request->withHeader('Content-Type', 'application/json');
        //$request = $request->withBody(GuzzleHttp\Psr7\stream_for(json_encode($requestBody)));
        $request = $request->withBody(GuzzleHttp\Psr7\Utils::streamFor(json_encode($requestBody)));


        $responseBody = $this->sendHttpRequest($request, $url)->getBody();
        return json_decode($responseBody)->{'id'};
    }

    public function sendHttpRequest($request, $url)
    {
        $client = new GuzzleHttp\Client();
        $ts = time();

        $request = $request->withHeader('X-App-Token', SUMSUB_APP_TOKEN);
        $request = $request->withHeader('X-App-Access-Sig', $this->createSignature($ts, $request->getMethod(), $url, $request->getBody()));
        $request = $request->withHeader('X-App-Access-Ts', $ts);
        
        // Reset stream offset to read body in `send` method from the start
        $request->getBody()->rewind();
        $response = null;
        try {
            $response = $client->send($request);
            // print_r($response);
            if ($response->getStatusCode() != 200 && $response->getStatusCode() != 201) {
                // https://developers.sumsub.com/api-reference/#errors
                // If an unsuccessful answer is received, please log the value of the "correlationId" parameter.
                // Then perhaps you should throw the exception. (depends on the logic of your code)
            }
        } catch (GuzzleHttp\Exception\GuzzleException $e) {
            print_r($e->getMessage());die;
            error_log($e);
        }

        return $response;
    }

    private function createSignature($ts, $httpMethod, $url, $httpBody)
    {
        return hash_hmac('sha256', $ts . strtoupper($httpMethod) . $url . $httpBody, SUMSUB_SECRET_KEY);
    }

    public function addDocument($applicantId)
        // https://developers.sumsub.com/api-reference/#adding-an-id-document
    {
        $metadata = ['idDocType' => 'PASSPORT', 'country' => 'GBR'];
        $file = __DIR__ . '/resources/images/sumsub-logo.png';

        $multipart = new MultipartStream([
            [
                "name" => "metadata",
                "contents" => json_encode($metadata)
            ],
            [
                'name' => 'content',
                'contents' => fopen($file, 'r')
            ],
        ]);

        $url = "/resources/applicants/" . $applicantId . "/info/idDoc";
        $request = new GuzzleHttp\Psr7\Request('POST', SUMSUB_TEST_BASE_URL . $url);
        $request = $request->withBody($multipart);

        return $this->sendHttpRequest($request, $url)->getHeader("X-Image-Id")[0];
    }

    // public function getApplicantStatus($applicantId)
    //     // https://developers.sumsub.com/api-reference/#getting-applicant-status-api
    // {
    //     $url = "/resources/applicants/" . $applicantId . "/requiredIdDocsStatus";
    //     $request = new GuzzleHttp\Psr7\Request('GET', SUMSUB_TEST_BASE_URL . $url);

    //     return $responseBody = $this->sendHttpRequest($request, $url)->getBody();
    //     return json_decode($responseBody);
    // }

    public function getAccessToken($externalUserId, $levelName)
        // https://developers.sumsub.com/api-reference/#access-tokens-for-sdks
    {
        $url = "/resources/accessTokens?userId=" . $externalUserId . "&levelName=" . $levelName;
        $request = new GuzzleHttp\Psr7\Request('POST', SUMSUB_TEST_BASE_URL . $url);

        return $this->sendHttpRequest($request, $url)->getBody();
    }

    public function getApplicationId($externalUserId)
        // https://developers.sumsub.com/api-reference/#getting-applicant-status-api
    {
        $url = "/resources/applicants/-;externalUserId=" . $externalUserId . "/one";
        $request = new GuzzleHttp\Psr7\Request('GET', SUMSUB_TEST_BASE_URL . $url);

        return $responseBody = $this->sendHttpRequest($request, $url)->getBody()->getContents();
        return json_decode($responseBody);
    }

    public function getApplicantStatus($applicantId)
        // https://developers.sumsub.com/api-reference/#getting-applicant-status-api
    {
        $url = "/resources/applicants/" . $applicantId . "/status";
        $request = new GuzzleHttp\Psr7\Request('GET', SUMSUB_TEST_BASE_URL . $url);

        return $responseBody = $this->sendHttpRequest($request, $url)->getBody();
        return json_decode($responseBody);
    }

}

// The description of the flow can be found here: https://developers.sumsub.com/api-flow/#api-integration-phases

// Such actions are presented below:
// 1) Creating an applicant
// 2) Adding a document to the applicant
// 3) Getting applicant status
// 4) Getting access token

$externalUserId = $_GET['id']??uniqid();
$levelName = 'basic-kyc-level';

$testObject = new AppTokenGuzzlePhpExample();

// $applicantId = $testObject->createApplicant($externalUserId, $levelName);
// echo "The applicant was successfully created: " . $applicantId . PHP_EOL;

// $imageId = $testObject->addDocument($applicantId);
// echo "Identifier of the added document: " . $imageId . PHP_EOL;

// $applicantStatusStr = $testObject->getApplicantStatus($applicantId);
// echo "Applicant status (json string): " . $applicantStatusStr;
if (isset($_GET['method']) && $_GET['method'] === 'token') {
    $accessTokenStr = $testObject->getAccessToken($externalUserId, $levelName);
    header('Content-Type: application/json; charset=utf-8');
    echo $accessTokenStr;
}
if (isset($_GET['method']) && $_GET['method'] === 'appicaltion_id') {
    $accessTokenStr = $testObject->getApplicationId($externalUserId);
    // echo $accessTokenStr; die;
    // print_r($accessTokenStr);die;
    header('Content-Type: application/json; charset=utf-8');
    echo $accessTokenStr;
}
if (isset($_GET['method']) && $_GET['method'] === 'appicaltion_status') {

    $accessTokenStr = $testObject->getApplicantStatus($externalUserId);
    // echo $accessTokenStr; die;
    // print_r($accessTokenStr);die;
    header('Content-Type: application/json; charset=utf-8');
    echo $accessTokenStr;
}

?>
